package cn.ibizlab.core.lite.extensions.domain;

import cn.ibizlab.core.lite.domain.MetaEntity;
import cn.ibizlab.core.lite.domain.MetaField;
import cn.ibizlab.core.lite.domain.MetaRelationship;
import com.fasterxml.jackson.annotation.JsonIgnore;
import lombok.Data;
import org.springframework.util.DigestUtils;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;

import java.util.*;

@Data
public class SysModel {

    private String systemid;

    @JsonIgnore
    private String systemname;

    private List<MetaEntity> entities;

    /**
     * 获取实体
     * @return
     */
    public Set<MetaEntity>  getEntity(){
        Set<MetaEntity> metaEntities=new HashSet<>();
        metaEntities.addAll(entities);
        return metaEntities;
    }

    /**
     * 获取实体属性
     * @return
     */
    public Set<MetaField>  getField(){
        Set<MetaField> fields=new HashSet<>();
        this.entities.forEach(entityNode->{
            List<MetaField> deField=entityNode.getFields();
            if(!ObjectUtils.isEmpty(deField)){
                deField.forEach(field->{
                    field.setEntityId(DigestUtils.md5DigestAsHex(String.format("%s||%s" ,systemid,entityNode.getEntityName()).getBytes()));
                });
                fields.addAll(deField);
            }
            }
        );
        return fields;
    }

    /**
     * 获取实体关系
     * @return
     */
    public Collection<MetaRelationship>  getRelation(){
        Map<String,MetaRelationship>  relations =new HashMap<>();
        this.entities.forEach(entityNode->{
            List<MetaRelationship> parentRelation= entityNode.getParentEntitys();
            List<MetaRelationship> subRelation= entityNode.getSubEntitys();
            if(!ObjectUtils.isEmpty(parentRelation)){
                addToRelations(relations,parentRelation);
            }
            if(!ObjectUtils.isEmpty(subRelation)){
                addToRelations(relations,subRelation);
            }
        });
        return relations.values();
    }

    private void addToRelations(Map<String,MetaRelationship> relations, List<MetaRelationship> entityRelations){
        entityRelations.forEach(entityRelation->{
            entityRelation.setEntityId(DigestUtils.md5DigestAsHex(String.format("%s||%s" ,systemid,entityRelation.getEntityName()).getBytes()));
            entityRelation.setRefEntityId(DigestUtils.md5DigestAsHex(String.format("%s||%s" ,systemid,entityRelation.getRefEntityName()).getBytes()));
            String relationId = entityRelation.getId();
            if( !StringUtils.isEmpty(relationId) &&  !relations.containsKey(relationId)){
                relations.put(relationId,entityRelation);
            }
        });
    }
}
